import enum
import inspect
import logging
from types import ModuleType
from typing import Any, List, Callable, Dict, Set

import threading
import wrapt

from _pytest.config import argparsing
from _pytest.fixtures import FixtureRequest
from pytest import fixture

from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support.abstract_event_listener import (
    AbstractEventListener,
)
from selenium.webdriver.support.event_firing_webdriver import (
    EventFiringWebDriver,
)

from . import screenshot_manager
from . import browser_console_manager
from .video_manager import ScreenRecorder
from . import common_utils
from . import config
from .config import EnhancedReportOperationFrequency as OpFreq, Parameter


logger = logging.getLogger(__name__)
logger.info("Loaded " + __file__)


# region Global Vars
__currently_applicable_reports: List[ModuleType] = []
__report_options: Dict[Parameter, Any] = {}
# endregion


# region Registering and retrieving config parameters
def pytest_addoption(parser: argparsing.Parser):
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)
    group = parser.getgroup("enhanced-report")
    config.register_with(group)


@fixture(scope="session", autouse=True)
def _report_options(request: FixtureRequest) -> Dict[Parameter, Any]:
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)
    __report_options.update(config.get_all_values(request))
    # logger.info("Report options: \n"+"\n".join([str(item) for item in __report_options.items()]))
    return __report_options


# endregion


# region Custom enums
class EnhancedReportAttachments(enum.Enum):
    # Tuple format: (type of attachment, minimal description for the attachment type)
    JS_LOG = ("text", "js log")
    SS = ("image", "screenshot")
    SS_WITH_HIGHLIGHT = ("image", "highlighted screenshot")
    VIDEO = ("video", "video")


class EnhancedReportLabels(enum.Enum):
    # Labels that would be used in the actual report(s)
    JS_LOGS = "Logs from browser console"
    JS_LOGS_WITH_DESC = f"{JS_LOGS} {{}}"
    SS = "Screenshot"
    SS_WITH_DESC = f"{SS} {{}}"
    VIDEO = "Video"
    VIDEO_WITH_DESC = f"{VIDEO} {{}}"

    def desc(self, desc: str = "") -> str:
        return self.value.format(desc)


class EnhancedReportTestState(enum.Enum):
    """Test states during which we expect a reportable event to occur."""

    BEFORE_TEST = "before test"
    CUSTOM_DURING_TEST = "custom during test"
    AFTER_TEST = "after test"
    BEFORE_UI_OPERATION = "before ui operation"
    AFTER_UI_OPERATION = "after ui operation"
    ERROR = "error"
    FAILED = "failed"
    PASSED = "passed"
    SKIPPED = "skipped"


# endregion


# region Rules and generic logic for attaching data to reports
"""
Maps each test state to a set of op. frequencies that allow data capture in that state.
"""
__state_and_allowed_frequencies: Dict[EnhancedReportTestState, Set[OpFreq]] = {
    EnhancedReportTestState.BEFORE_TEST: {OpFreq.ALWAYS},
    EnhancedReportTestState.BEFORE_UI_OPERATION: {
        OpFreq.ALWAYS,
        OpFreq.EACH_UI_OPERATION,
    },
    EnhancedReportTestState.AFTER_UI_OPERATION: {
        OpFreq.ALWAYS,
        OpFreq.EACH_UI_OPERATION,
    },
    EnhancedReportTestState.AFTER_TEST: {
        OpFreq.ALWAYS,
        OpFreq.END_OF_EACH_TEST,
    },
    EnhancedReportTestState.ERROR: {
        OpFreq.ALWAYS,
        OpFreq.END_OF_EACH_TEST,
        OpFreq.FAILED_TEST_ONLY,
    },
    EnhancedReportTestState.FAILED: {
        OpFreq.ALWAYS,
        OpFreq.END_OF_EACH_TEST,
        OpFreq.FAILED_TEST_ONLY,
    },
    EnhancedReportTestState.PASSED: {OpFreq.ALWAYS, OpFreq.END_OF_EACH_TEST},
    # planned for the future. not supported atm
    EnhancedReportTestState.CUSTOM_DURING_TEST: {},
    # a skipped test is not expected to have an active browser session to extract info from
    EnhancedReportTestState.SKIPPED: {},
}

"""
Maps each attachment type to the config param for that type.
"""
__attachment_and_config: Dict[EnhancedReportAttachments, Parameter] = {
    EnhancedReportAttachments.JS_LOG: Parameter.JS_LOG_FREQUENCY,
    EnhancedReportAttachments.SS: Parameter.SS_FREQUENCY,
    EnhancedReportAttachments.SS_WITH_HIGHLIGHT: Parameter.SS_HIGHLIGHT_ELEMENT,
    EnhancedReportAttachments.VIDEO: Parameter.VIDEO_ENABLED,
}


def __can_record(
    attachment_type: EnhancedReportAttachments,
    current_state: EnhancedReportTestState,
):
    # Get the config value for the current type
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)
    param_value = __report_options[__attachment_and_config[attachment_type]]
    logger.info(f"current param_value {param_value}")
    if (
        not param_value
        or param_value not in __state_and_allowed_frequencies[current_state]
    ):
        return False

    can_record: bool = True

    if (
        attachment_type == EnhancedReportAttachments.VIDEO and not param_value
    ):  # Video is not enabled
        # logger.debug("Rejection reason: Video is not enabled")
        can_record = False
    elif attachment_type == EnhancedReportAttachments.SS_WITH_HIGHLIGHT:
        # With highlighted screenshots, check both the specific param and the screenshot frequency
        if (
            not param_value
            or __report_options[Parameter.SS_FREQUENCY]
            not in __state_and_allowed_frequencies[current_state]
        ):
            # logger.debug("Rejection reason: Highlighted screenshots are either not enabled, or they're not allowed in this state")
            can_record = False

    # logger.debug(f"Recording {attachment_type} is {'' if can_record else 'NOT '}allowed in state - {current_state}")
    return can_record


def __report_data_handler(
    attachment_type: EnhancedReportAttachments, name: str, value: str, **kwargs
):
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)
    for report_mod in __currently_applicable_reports:
        try:
            getattr(report_mod, f"attach_{attachment_type.value[0]}")(
                name, value, **kwargs
            )
        except Exception as e:
            logger.error(
                f"Error while attaching {attachment_type.value[1]} to report {report_mod}: {e}"
            )


# endregion


# region Convenience functions for attaching data to reports
def __capture_ss(
    attachment_type: EnhancedReportAttachments,
    state: EnhancedReportTestState,
    scenario_name,
    name,
    driver,
    element=None,
):
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)
    if not __can_record(attachment_type, state):
        return

    if (
        attachment_type == EnhancedReportAttachments.SS_WITH_HIGHLIGHT
        and element
    ):
        path: str = screenshot_manager.get_highlighted_screenshot(
            element, name, scenario_name, __report_options, driver
        )
    else:
        path: str = screenshot_manager.get_screenshot(
            name, scenario_name, __report_options, driver
        )

    logger.debug(f"Captured screenshot path: {path}")
    if path:  # there is a chance that a screenshot was not captured
        __report_data_handler(
            attachment_type, EnhancedReportLabels.SS_WITH_DESC.desc(name), path
        )


def __capture_js_logs(state: EnhancedReportTestState, driver, label: str):
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)
    if not __can_record(EnhancedReportAttachments.JS_LOG, state):
        return

    logs = browser_console_manager.get_js_logs(driver)
    __report_data_handler(
        EnhancedReportAttachments.JS_LOG,
        EnhancedReportLabels.JS_LOGS_WITH_DESC.desc(label),
        logs,
    )


# endregion


# region Report agnostic setup and teardown
@fixture(scope="session", autouse=True)
def _global_config(request, _report_options):
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)

    # region Teardown
    def remove_test_dir():
        try:
            if not _report_options[Parameter.VIDEO_KEEP_FILES]:
                common_utils.delete_dir(_report_options[Parameter.VIDEO_DIR])

            if not _report_options[Parameter.SS_KEEP_FILES]:
                common_utils.delete_dir(_report_options[Parameter.SS_DIR])
            else:
                common_utils.delete_files(_report_options[Parameter.SS_DIR])
        except Exception as error:
            logger.error(f"Error occurred while cleaning up: {error}")

    request.addfinalizer(remove_test_dir)
    # endregion

    # region Setup
    # create the screenshot directory if it doesn't exist
    if _report_options[Parameter.SS_FREQUENCY] != OpFreq.NEVER:
        common_utils.mkdir(_report_options[Parameter.SS_DIR])

    # create the video directory if it doesn't exist
    if _report_options[Parameter.VIDEO_ENABLED]:
        common_utils.mkdir(_report_options[Parameter.VIDEO_DIR])
    # endregion


# endregion


# region Local utils
def __get_all_module_names_in_relative_path(path: str) -> List[str]:
    import pkgutil
    import os

    # Getting the absolute path to the current file, and removing the file name to get the current directory
    lib_path: str = os.path.abspath(__file__)[: -len("core.py")]
    return [name for _, name, _ in pkgutil.iter_modules([lib_path + path])]


def __is_pytest_plugin_installed(
    request: FixtureRequest, plugin_name: str
) -> bool:
    return request.config.pluginmanager.has_plugin(plugin_name)


# endregion


# region handling report lib integrations
@fixture(scope="session", autouse=True)
def _reports(request: FixtureRequest, _report_options) -> List[ModuleType]:
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)

    # region Teardown
    def report_specific_cleanup():
        for report_mod in __currently_applicable_reports:
            try:
                logger.debug(
                    "Calling session scoped teardown for " + report_mod.__name__
                )
                report_mod.perform_session_cleanup(request, _report_options)
            except Exception as e:
                logger.error(
                    f"Error while performing session level cleanup for report {report_mod.__name__}: {e}"
                )

    request.addfinalizer(report_specific_cleanup)
    # endregion

    # region Setup
    supported_reports: List[str] = __get_all_module_names_in_relative_path(
        "report_libs"
    )
    logger.debug(f"supported reports - {supported_reports}")

    import importlib

    for report_name in supported_reports:
        try:
            if __is_pytest_plugin_installed(request, report_name):
                logger.debug(
                    f"pytest plugin '{report_name}' is installed. Importing corresponding report lib"
                )
                mod = importlib.import_module(
                    f".report_libs.{report_name}", package="enhanced_reports"
                )
                __currently_applicable_reports.append(mod)
                logger.debug("Calling session scoped setup for " + report_name)
                mod.perform_session_setup(request, _report_options)
            else:
                logger.debug(
                    f"pytest plugin '{report_name}' is not installed. Ignoring module"
                )
        except Exception as e:
            logger.error(
                f"Failed to load or setup session level enhancements for report {report_name}: {e}"
            )

    return __currently_applicable_reports
    # endregion


@fixture(autouse=True)
def _reports_function_scope(request: FixtureRequest, _reports, _report_options):
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)

    # region Teardown
    def report_specific_cleanup():
        for report_mod in _reports:
            try:
                logger.debug(
                    "Calling function scoped teardown for "
                    + report_mod.__name__
                )
                report_mod.perform_function_cleanup(request, _report_options)
            except Exception as e:
                logger.error(
                    f"Error while performing cleanup for function scope for report {report_mod.__name__}: {e}"
                )

    request.addfinalizer(report_specific_cleanup)
    # endregion

    # region Setup
    for report_mod in _reports:
        try:
            logger.debug(
                "Calling function scoped setup for " + report_mod.__name__
            )
            report_mod.perform_function_setup(request, _report_options)
        except Exception as e:
            logger.error(
                f"Error while performing function level setup for report {report_mod.__name__}: {e}"
            )
    # endregion


# endregion


# region Gather test metadata
@fixture
def _scenario_name(request: FixtureRequest) -> str:
    return __scenario_name_supplier(request)()


def __scenario_name_supplier(request: FixtureRequest) -> Callable[[], str]:
    return lambda: request.node.nodeid.split("/")[-1].replace("::", " - ")


# endregion


# region WebDriver acquisition
@fixture
def _local_driver() -> Dict[str, WebDriver]:
    return {"driver": None}  # type: ignore


@fixture  # TODO: test this with a session scoped fixture in the test framework for instantiating the driver
def enhance_driver(request, _report_options, _local_driver):
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)

    def _enhanced_driver_getter(driver: WebDriver):

        _local_driver["driver"] = driver

        return EventFiringWebDriver(
            driver,
            WebDriverEventListener(
                _report_options,
                __capture_ss,
                __capture_js_logs,
                __scenario_name_supplier(request),
            ),
        )

    return _enhanced_driver_getter


# endregion


# region Video recording
@fixture(autouse=True)
def _video_capture(
    request, _scenario_name, _report_options: Dict[Parameter, Any]
):
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)
    screen_recorder = {}
    recorder_thread = None
    if __can_record(
        EnhancedReportAttachments.VIDEO, EnhancedReportTestState.BEFORE_TEST
    ):
        logger.debug("Initializing video recording")
        screen_recorder = ScreenRecorder(
            directory=_scenario_name,
            video_store=_report_options[Parameter.VIDEO_DIR],
        )
        common_utils.mkdir(_report_options[Parameter.VIDEO_DIR])

        driver = request.getfixturevalue("_local_driver")["driver"]
        recorder_thread = threading.Thread(
            target=screen_recorder.start_capturing,
            name="Recorder",
            args=[driver],
        )

        logger.debug("Starting video recording")
        recorder_thread.start()

        # this is inside the if block intentionally. only need to stop the recording if it was started
        request.addfinalizer(
            lambda: screen_recorder.stop_recording_and_stitch_video(
                _report_options, recorder_thread, _scenario_name, _scenario_name
            )
        )

    yield  # dummy yield in order to prevent the fixture from being run from the beginning during teardown


# endregion


# region Screenshot & js log capture
def pytest_bdd_step_error(request, feature, scenario, step, step_func):
    current_state = EnhancedReportTestState.ERROR
    op_name = "after action chain"

    driver = request.getfixturevalue("_local_driver")["driver"]
    scenario_name = request.getfixturevalue("_scenario_name")

    if __can_record(EnhancedReportAttachments.SS, current_state):
        __capture_ss(
            EnhancedReportAttachments.SS,
            current_state,
            scenario_name,
            op_name,
            driver,
        )

    if __can_record(EnhancedReportAttachments.JS_LOG, current_state):
        __capture_js_logs(current_state, driver, op_name)


def pytest_bdd_after_scenario(request, feature, scenario):
    current_state = EnhancedReportTestState.AFTER_TEST
    op_name = "after test"

    driver = request.getfixturevalue("_local_driver")["driver"]
    scenario_name = request.getfixturevalue("_scenario_name")

    if __can_record(EnhancedReportAttachments.SS, current_state):
        __capture_ss(
            EnhancedReportAttachments.SS,
            current_state,
            scenario_name,
            op_name,
            driver,
        )

    if __can_record(EnhancedReportAttachments.JS_LOG, current_state):
        __capture_js_logs(current_state, driver, op_name)


@fixture(scope="session", autouse=True)
def _create_wrappers(request: FixtureRequest, _report_options):
    logger.debug("Entered " + inspect.currentframe().f_code.co_name)
    current_state = EnhancedReportTestState.AFTER_UI_OPERATION
    op_name = "after action chain"

    if not __can_record(EnhancedReportAttachments.SS, current_state):
        return

    @wrapt.patch_function_wrapper(ActionChains, "perform")
    def wrap_action_chains_perform_method(wrapped, instance, args, kwargs):
        # here, wrapped is the original perform method in ActionChains
        # instance is `self` (it is not the case for classmethods though),
        # args and kwargs are a tuple and a dict respectively.

        wrapped(*args, **kwargs)  # note it is already bound to the instance

        scenario_name = request.getfixturevalue("_scenario_name")
        __capture_ss(
            EnhancedReportAttachments.SS,
            current_state,
            scenario_name,
            op_name,
            instance._driver,
        )
        __capture_js_logs(current_state, instance._driver, op_name)


class WebDriverEventListener(AbstractEventListener):
    def __init__(
        self,
        _report_options: Dict[Parameter, Any],
        ss_handler: Callable,
        js_log_handler: Callable,
        scenario_name_supplier: Callable[[], str],
    ):
        # just store all arguments as instance vars
        self.__report_options: Dict[Parameter, Any] = _report_options
        self.__capture_ss: Callable = ss_handler
        self.__capture_js_logs: Callable = js_log_handler
        self.__scenario_name_supplier: Callable[
            [], str
        ] = scenario_name_supplier

    def after_navigate_to(self, url, driver: WebDriver):
        current_state = EnhancedReportTestState.AFTER_UI_OPERATION
        op_name: str = f"Navigation to {url}"
        self.__capture_ss(
            EnhancedReportAttachments.SS,
            current_state,
            self.__scenario_name_supplier(),
            op_name,
            driver,
        )
        self.__capture_js_logs(current_state, driver, op_name)

    def before_click(self, element, driver):
        current_state = EnhancedReportTestState.BEFORE_UI_OPERATION
        op_name: str = "before click"
        self.__capture_ss(
            EnhancedReportAttachments.SS_WITH_HIGHLIGHT,
            current_state,
            self.__scenario_name_supplier(),
            op_name,
            driver,
            element=element,
        )

    def after_click(self, element, driver):
        current_state = EnhancedReportTestState.AFTER_UI_OPERATION
        op_name: str = "after click"
        self.__capture_ss(
            EnhancedReportAttachments.SS,
            current_state,
            self.__scenario_name_supplier(),
            op_name,
            driver,
        )
        self.__capture_js_logs(current_state, driver, op_name)

    def before_change_value_of(self, element, driver):
        current_state = EnhancedReportTestState.BEFORE_UI_OPERATION
        op_name = "before keyboard input"
        self.__capture_ss(
            EnhancedReportAttachments.SS_WITH_HIGHLIGHT,
            current_state,
            self.__scenario_name_supplier(),
            op_name,
            driver,
            element=element,
        )

    def after_change_value_of(self, element, driver):
        current_state = EnhancedReportTestState.AFTER_UI_OPERATION
        op_name = "after keyboard input"
        self.__capture_ss(
            EnhancedReportAttachments.SS,
            current_state,
            self.__scenario_name_supplier(),
            op_name,
            driver,
        )
        self.__capture_js_logs(
            EnhancedReportTestState.AFTER_UI_OPERATION, driver, op_name
        )

    def after_execute_script(self, script, driver):
        current_state = EnhancedReportTestState.AFTER_UI_OPERATION
        op_name = "after JS execution"
        self.__capture_ss(
            EnhancedReportAttachments.SS,
            current_state,
            self.__scenario_name_supplier(),
            op_name,
            driver,
        )
        self.__capture_js_logs(current_state, driver, op_name)

    def after_navigate_back(self, driver):
        self.__capture_js_logs(
            EnhancedReportTestState.AFTER_UI_OPERATION,
            driver,
            "after navigating back",
        )


# endregion



from setuptools import setup

NAME = "pytest-enhanced-reports"
VERSION = "1.2.1"

INSTALL_REQUIRES = [
    "selenium >=4.0.0, <4.1.4",
    "pytest >=6.2.5, < 7.0.0",
    "pytest-bdd >=5.0.0, < 6.0.0",
    "python-dotenv >=0.15.0, <1.0.0",
    "Pillow >=8.4.0, <9.0.0",
    "wrapt >=1.14.1, <2.0.0",
    "opencv-python",
]

setup(
    name=NAME,
    version=VERSION,
    description="Enhanced test reports for pytest",
    long_description="A enhanced-allure-report to improve allure report by adding screenshots, videos, browser's "
    "outputs",
    author="NewPage Solutions",
    author_email="InnovationDesk@newpage.io",
    license="MIT",
    packages=["enhanced_reports", "enhanced_reports.report_libs"],
    install_requires=INSTALL_REQUIRES,
    entry_points={"pytest11": ["enhanced_reports = enhanced_reports.core"]},
    classifiers=["Framework :: Pytest"],
)



import logging
import os
import re
from typing import Tuple
from PIL import Image

logger = logging.getLogger(__name__)
logger.info("Loaded " + __file__)


def get_resized_resolution(width, height, resize_factor) -> Tuple[int, int]:
    new_width = int(width * resize_factor)
    new_height = int(height * resize_factor)
    return new_width, new_height


def mkdir(dir_name):
    os.makedirs(dir_name, exist_ok=True)


def delete_dir(dir_path):
    if os.path.isdir(dir_path):
        for f in os.listdir(dir_path):
            if os.path.isdir(os.path.join(dir_path, f)):
                delete_dir(os.path.join(dir_path, f))
            else:
                os.remove(os.path.join(dir_path, f))
        os.rmdir(dir_path)
        logger.debug(f"Deleted the dir '{dir_path}'")


def delete_files(img_dir, file_name=None, extension="png"):
    if file_name:
        os.remove(os.path.join(img_dir, file_name))
        return

    if not os.path.isdir(img_dir):
        logger.warning(f"Directory '{img_dir}' does not exist")
        return

    for f in os.listdir(img_dir):
        if f.endswith(extension):
            os.remove(os.path.join(img_dir, f))
    logger.debug(f"Files with extension {extension} deleted from {img_dir}")


def clean_filename(value: str) -> str:
    # remove the undesirable characters
    return re.sub(r"\W", "_", value)


def fail_silently(func):
    """Decorator that makes sure that any errors/exceptions do not get outside the plugin"""

    def wrapped_func(*args, **kws):
        try:
            return func(*args, **kws)
        except Exception as e:
            logger.error(f"Error in {func.__name__}: {e}")

    return wrapped_func


def get_image_resolution(directory, file_name=None):
    """get the original resolution of an image"""
    if not file_name:
        img = Image.open(
            os.path.join(
                directory,
                [f for f in os.listdir(directory) if f.endswith(".png")][0],
            )
        )
    else:
        img = Image.open(os.path.join(directory, file_name))
    return img.width, img.height



import os
from typing import Dict, Any, Tuple

import allure
from allure_commons.types import AttachmentType
from PIL import Image
import cv2

from . import common_utils
from .common_utils import get_image_resolution
from .config import Parameter

import logging

logger = logging.getLogger(__name__)
logger.info("Loaded " + __file__)


class ScreenRecorder:
    def __init__(self, directory: str = "temp/", video_store: str = "videos"):
        self.stop = False
        self.__directory = directory  # This directory will be used to save the frames temporarily
        self.__video_store = (
            video_store  # This will be used to save the recorded video
        )
        self.__desired_resolution: Tuple[int, int] = None
        self.__resize_factor: float = None

    def start_capturing(self, driver):
        """This method will start capturing images and saving them on disk under /video folder
        These images will later be used to stich together into a video"""
        try:
            count = 0
            common_utils.mkdir(self.__directory)
            while True:
                driver.save_screenshot(
                    self.__directory + f"/vid_frame{count}.png"
                )
                count += 1
                if self.stop:
                    logger.debug(
                        f"Stopping screenshot capture for video. Captured {count} screenshots."
                    )
                    break
        except Exception as error:
            logger.error(
                "An Exception occurred while taking screenshot. " + str(error)
            )

    def create_video_from_images(
        self, scenario_info, location, video_size: tuple, frame_rate: int
    ):
        """This method will stitch the images under /video directory into a video"""
        fourcc = cv2.VideoWriter_fourcc(*"vp09")
        video_name = f"{location}/{scenario_info}.webm"
        video = cv2.VideoWriter(video_name, fourcc, int(frame_rate), video_size)
        image_files = [
            f for f in os.listdir(self.__directory) if f.endswith(".png")
        ]
        image_files = sorted(
            image_files, key=lambda x: int(os.path.splitext(x)[0])
        )
        for img_file in image_files:
            video.write(
                cv2.resize(
                    cv2.imread(os.path.join(self.__directory, img_file)),
                    video_size,
                )
            )
        video.release()

        logger.debug(
            "Video recording completed. [Video Size: "
            + str(video_size)
            + " - Frame Rate: "
            + str(frame_rate)
            + "]"
        )

        return video_name

    def stop_recording_and_stitch_video(
        self,
        report_options: Dict[Parameter, Any],
        recorder_thread,
        scenario_info,
        attachment_name,
    ):
        try:
            self.stop = True
            recorder_thread.join()
            video_resize_info = self.get_video_resize_resolution(report_options)
            file_name = self.create_video_from_images(
                scenario_info,
                self.__directory,
                video_resize_info,
                report_options[Parameter.VIDEO_FRAME_RATE],
            )
            allure.attach.file(
                file_name,
                name=attachment_name,
                attachment_type=AttachmentType.WEBM,
            )

            if report_options[Parameter.VIDEO_KEEP_FILES]:
                original_size = get_image_resolution(self.__directory)
                self.create_video_from_images(
                    scenario_info,
                    self.__video_store,
                    original_size,
                    report_options[Parameter.VIDEO_FRAME_RATE],
                )

        except Exception as error:
            logger.error(
                "An Exception occurred while stitching video. " + str(error)
            )
        finally:
            # Now clean the images directory
            common_utils.delete_dir(self.__directory)

    def get_video_resize_resolution(self, report_options: Dict[Parameter, Any]):
        try:
            directory = self.__directory
            desired_resolution = (
                self.__desired_resolution
                if self.__desired_resolution
                else (
                    report_options[Parameter.VIDEO_WIDTH],
                    report_options[Parameter.VIDEO_HEIGHT],
                )
            )
            resize_factor = (
                self.__resize_factor
                if self.__resize_factor
                else report_options[Parameter.VIDEO_RESIZE_PERCENT] / 100
            )

            if desired_resolution == (0, 0):
                # use the resize factor to calculate the desired resolution from actual resolution of the image
                img = Image.open(
                    os.path.join(
                        directory,
                        [
                            f
                            for f in os.listdir(directory)
                            if f.endswith(".png")
                        ][0],
                    )
                )
                desired_resolution = common_utils.get_resized_resolution(
                    img.width, img.height, resize_factor
                )

            return desired_resolution
        except Exception as error:
            logger.error(
                "An Exception occurred while fetching video resize resolution. "
                + str(error)
            )
            # Now clean the images in temp directory as video stitching has failed
            common_utils.delete_dir(self.__directory)
